/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/

#include "ER-EPD075-1.h"
#include "Debug.h"

/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
static void EPD_075_1_Reset(void)
{
    DEV_Digital_Write(EPD_RST_PIN, 1);
    DEV_Delay_ms(200);
    DEV_Digital_Write(EPD_RST_PIN, 0);
    DEV_Delay_ms(10);
    DEV_Digital_Write(EPD_RST_PIN, 1);
    DEV_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
     Reg : Command register
******************************************************************************/
static void EPD_075_1_SendCommand(UBYTE Reg)
{
    DEV_Digital_Write(EPD_DC_PIN, 0);
    DEV_Digital_Write(EPD_CS_PIN, 0);
    DEV_SPI_WriteByte(Reg);
    DEV_Digital_Write(EPD_CS_PIN, 1);
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
static void EPD_075_1_SendData(UBYTE Data)
{
    DEV_Digital_Write(EPD_DC_PIN, 1);
    DEV_Digital_Write(EPD_CS_PIN, 0);
    DEV_SPI_WriteByte(Data);
    DEV_Digital_Write(EPD_CS_PIN, 1);
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD_075_1_ReadBusy(void)
{
    UBYTE busy;
    Debug("e-Paper busy\r\n");
    do {
        EPD_075_1_SendCommand(0x71);
        busy = DEV_Digital_Read(EPD_BUSY_PIN);
        busy =!(busy & 0x01);
    } while(busy);
    Debug("e-Paper busy release\r\n");
}

/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
static void EPD_075_1_TurnOnDisplay(void)
{
    EPD_075_1_SendCommand(0x04); // POWER ON
    EPD_075_1_ReadBusy();
    EPD_075_1_SendCommand(0x12); // display refresh
    DEV_Delay_ms(100);
    EPD_075_1_ReadBusy();
}

/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
void EPD_075_1_Init(void)
{
    EPD_075_1_Reset();

    EPD_075_1_SendCommand(0x01); // POWER_SETTING
    EPD_075_1_SendData(0x37);
    EPD_075_1_SendData(0x00);

    EPD_075_1_SendCommand(0x00); // PANEL_SETTING
    EPD_075_1_SendData(0xCF);
    EPD_075_1_SendData(0x08);

    EPD_075_1_SendCommand(0x30); // PLL_CONTROL
    EPD_075_1_SendData(0x3A); // PLL:  0-15:0x3C, 15+:0x3A

    EPD_075_1_SendCommand(0x82); // VCM_DC_SETTING
    EPD_075_1_SendData(0x28); //all temperature  range

    EPD_075_1_SendCommand(0x06); // BOOSTER_SOFT_START
    EPD_075_1_SendData (0xc7);
    EPD_075_1_SendData (0xcc);
    EPD_075_1_SendData (0x15);

    EPD_075_1_SendCommand(0x50); // VCOM AND DATA INTERVAL SETTING
    EPD_075_1_SendData(0x77);

    EPD_075_1_SendCommand(0x60); // TCON_SETTING
    EPD_075_1_SendData(0x22);

    EPD_075_1_SendCommand(0x65); // FLASH CONTROL
    EPD_075_1_SendData(0x00);

    EPD_075_1_SendCommand(0x61);  // TCON_RESOLUTION
    EPD_075_1_SendData(EPD_075_1_WIDTH >> 8); // source 640
    EPD_075_1_SendData(EPD_075_1_WIDTH & 0xff);
    EPD_075_1_SendData(EPD_075_1_HEIGHT >> 8); // gate 384
    EPD_075_1_SendData(EPD_075_1_HEIGHT & 0xff);

    EPD_075_1_SendCommand(0xe5); // FLASH MODE
    EPD_075_1_SendData(0x03);
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD_075_1_Clear(void)
{
    UWORD Width, Height,j,i;UBYTE k;
    Width = (EPD_075_1_WIDTH % 8 == 0)? (EPD_075_1_WIDTH / 8): (EPD_075_1_WIDTH / 8 + 1);
    Height = EPD_075_1_HEIGHT;

    EPD_075_1_SendCommand(0x10);
    for ( j = 0; j < Height; j++) {
        for ( i = 0; i < Width; i++) {
            for( k = 0; k < 4; k++) {
                EPD_075_1_SendData(0x33);
            }
        }
    }

    EPD_075_1_TurnOnDisplay();
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD_075_1_Display(const UBYTE *blackimage, const UBYTE *ryimage)
{
    UBYTE Data_Black, Data_RY, Data,k;
    UDOUBLE i, j, Width, Height;
    Width = (EPD_075_1_WIDTH % 8 == 0)? (EPD_075_1_WIDTH / 8 ): (EPD_075_1_WIDTH / 8 + 1);
    Height = EPD_075_1_HEIGHT;

    EPD_075_1_SendCommand(0x10);
    for (j = 0; j < Height; j++) {
        for (i = 0; i < Width; i++) {
            Data_Black = blackimage[i + j * Width];
            Data_RY = ryimage[i + j * Width]; // red or yellow
            for(k = 0; k < 8; k++) {
                if ((Data_RY & 0x80) == 0x00) {
                    Data = 0x04;                //red
                } else if ((Data_Black & 0x80) == 0x00) {
                    Data = 0x00;               //black
                } else {
                    Data = 0x03;                //white
                }
                Data = (Data << 4) & 0xFF;
                Data_Black = (Data_Black << 1) & 0xFF;
                Data_RY = (Data_RY << 1) & 0xFF;
                k += 1;

                if((Data_RY & 0x80) == 0x00) {
                    Data |= 0x04;              //red
                } else if ((Data_Black & 0x80) == 0x00) {
                    Data |= 0x00;              //black
                } else {
                    Data |= 0x03;              //white
                }
                Data_Black = (Data_Black << 1) & 0xFF;
                Data_RY = (Data_RY << 1) & 0xFF;
                EPD_075_1_SendData(Data);
            }
        }
    }

    EPD_075_1_TurnOnDisplay();
}

void EPD_075_1_DisplayHalfScreen(const UBYTE *blackimage, const UBYTE *ryimage)
{
    UBYTE Data_Black, Data_RY, Data,k;
    UDOUBLE i, j, Width, Height;
    Width = (EPD_075_1_WIDTH % 8 == 0)? (EPD_075_1_WIDTH / 8 ): (EPD_075_1_WIDTH / 8 + 1);
    Height = EPD_075_1_HEIGHT;

    EPD_075_1_SendCommand(0x10);
    for (j = 0; j < Height / 2; j++) {
        for (i = 0; i < Width; i++) {
            Data_Black = blackimage[i + j * Width];
            Data_RY = ryimage[i + j * Width]; // red or yellow
            for( k = 0; k < 8; k++) {
                if ((Data_RY & 0x80) == 0x00) {
                    Data = 0x04;                //red
                } else if ((Data_Black & 0x80) == 0x00) {
                    Data = 0x00;               //black
                } else {
                    Data = 0x03;                //white
                }
                Data = (Data << 4) & 0xFF;
                Data_Black = (Data_Black << 1) & 0xFF;
                Data_RY = (Data_RY << 1) & 0xFF;
                k += 1;

                if((Data_RY & 0x80) == 0x00) {
                    Data |= 0x04;              //red
                } else if ((Data_Black & 0x80) == 0x00) {
                    Data |= 0x00;              //black
                } else {
                    Data |= 0x03;              //white
                }
                Data_Black = (Data_Black << 1) & 0xFF;
                Data_RY = (Data_RY << 1) & 0xFF;
                EPD_075_1_SendData(Data);
            }
        }
    }
		
		for (j = 0; j < Height / 2; j++) {
        for (i = 0; i < Width; i++) {
           for( k = 0; k < 4; k++) {
                EPD_075_1_SendData(Data);
            } 
        }
    }

    EPD_075_1_TurnOnDisplay();
}


/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD_075_1_Sleep(void)
{
    EPD_075_1_SendCommand(0x02); // POWER_OFF
    EPD_075_1_ReadBusy();
    EPD_075_1_SendCommand(0x07); // DEEP_SLEEP
    EPD_075_1_SendData(0XA5);
}
